import datetime
import re

from pyspark.sql import SparkSession
from pyspark.sql.types import StructType, LongType, StructField


def create_sk(sparkSession: SparkSession, df, offset, colName):
    new_schema = StructType(
        [StructField(colName, LongType(), True)]
        + df.schema.fields
    )

    zipped_rdd = df.rdd.zipWithIndex()

    new_rdd = zipped_rdd.map(lambda args: ([args[1] + offset] + list(args[0])))
    return sparkSession.createDataFrame(new_rdd, new_schema)


def valid_date(date):
    # format
    date_pattern = re.compile(r'\d{4}-\d{2}-\d{2}')
    if not date_pattern.match(date):
        return 'unknown'

    # real date
    date_parts = date.split('-')
    try:
        datetime.date(int(date_parts[0]), int(date_parts[1]), int(date_parts[2]))
    except ValueError:
        return 'unknown'

    return date


def valid_timestamp(timestamp):
    timestamp_pattern = re.compile(r'\d{4}-\d{2}-\d{2} ([0-1]\d|2[0-3]):[0-5]\d:[0-5]\d')
    if not timestamp_pattern.match(timestamp):
        return 'unknown'

    return timestamp
