import logging

import mysql.connector
from pyspark.sql import SparkSession

from common.constants import MASTER_ADDR, DWH_DIR_ADDR, LOG_FILE, MYSQL_DB, RESULT_LOG_FILE


def get_spark(job_name):
    sparkSession = SparkSession.builder \
        .appName(f"ETL_{job_name}") \
        .master(MASTER_ADDR) \
        .config("spark.sql.warehouse.dir", DWH_DIR_ADDR) \
        .enableHiveSupport() \
        .getOrCreate()

    return sparkSession


def setup_logger(name, log_file, formatter):
    handler = logging.FileHandler(log_file)
    handler.setFormatter(formatter)

    logger = logging.getLogger(name)
    logger.setLevel(logging.INFO)
    logger.addHandler(handler)

    return logger


def get_logger(job=""):
    formatter = logging.Formatter('%(asctime)s %(levelname)s {}: %(message)s'.format(job),
                                  '%Y-%m-%d %H:%M:%S')
    logger = setup_logger(f'{job}_logger', LOG_FILE, formatter)
    return logger


def get_result_logger():
    formatter = logging.Formatter('%(message)s')
    logger = setup_logger(f'result_logger', RESULT_LOG_FILE, formatter)
    return logger


def get_mysql():
    mysql_db = mysql.connector.connect(host='localhost',
                                       database=MYSQL_DB,
                                       user='root',
                                       password='12345')
    if mysql_db.is_connected():
        return mysql_db


def stop_mysql(mysql_db):
    if mysql_db is not None and mysql_db.is_connected():
        mysql_db.close()
