from common.constants import *
from common.tools import get_mysql, stop_mysql


class DAFunctions:
    def __init__(self):
        self.mysql_db = get_mysql()
        self.cursor = self.mysql_db.cursor()

    def init(self):
        cmd_log_drop_query = f"""DROP TABLE IF EXISTS {CMD_LOG}"""

        cmd_log_create_query = f"""CREATE TABLE {CMD_LOG} ( 
                                etl_cmd varchar(50) NOT NULL,
                                result BOOLEAN NOT NULL,
                                PRIMARY KEY (etl_cmd)) """

        cmd_log_insert_query = f"INSERT INTO {CMD_LOG} (etl_cmd, result) VALUES (%s, %s)"
        cmd_vals = [(f"{USER_DIM}{EXT_OP}", False),
                    (f"{TRACK_DIM}{EXT_OP}", False),
                    (f"{STREAMS_FACT}{EXT_OP}", False),
                    (f"{USER_DIM}{TL_OP}", False),
                    (f"{TRACK_DIM}{TL_OP}", False),
                    (f"{STREAMS_FACT}{TL_OP}", False),
                    (f"{USER_DIM}{SWAP_OP}", False),
                    (f"{TRACK_DIM}{SWAP_OP}", False),
                    (f"{STREAMS_FACT}{SWAP_OP}", False)]

        self.cursor.execute(cmd_log_drop_query)

        self.cursor.execute(cmd_log_create_query)

        self.cursor.executemany(cmd_log_insert_query, cmd_vals)

        self.create_config_log()

        self.mysql_db.commit()

        self.mysql_disconnect()

    def create_config_log(self):
        self.cursor.execute(f"SHOW TABLES LIKE '{CONFIG_LOG}'")
        result = self.cursor.fetchone()
        if not result:
            self.cursor.execute(f"""CREATE TABLE {CONFIG_LOG} ( 
                                    etl_cmd varchar(50) NOT NULL,
                                    prev_step varchar(50) NOT NULL)""")

            config_log_insert_query = f"INSERT INTO {CONFIG_LOG} (etl_cmd, prev_step) VALUES (%s, %s)"
            config_vals = [(f"{USER_DIM}{TL_OP}", f"{USER_DIM}{EXT_OP}"),
                           (f"{USER_DIM}{SWAP_OP}", f"{USER_DIM}{TL_OP}"),
                           (f"{TRACK_DIM}{TL_OP}", f"{TRACK_DIM}{EXT_OP}"),
                           (f"{TRACK_DIM}{SWAP_OP}", f"{TRACK_DIM}{TL_OP}"),
                           (f"{STREAMS_FACT}{TL_OP}", f"{STREAMS_FACT}{EXT_OP}"),
                           (f"{STREAMS_FACT}{TL_OP}", f"{USER_DIM}{TL_OP}"),
                           (f"{STREAMS_FACT}{TL_OP}", f"{TRACK_DIM}{TL_OP}"),
                           (f"{STREAMS_FACT}{SWAP_OP}", f"{STREAMS_FACT}{TL_OP}")]

            self.cursor.executemany(config_log_insert_query, config_vals)

    def log_cmd_result(self, cmd):
        query = f"UPDATE {CMD_LOG} SET result = TRUE WHERE etl_cmd = '{cmd}'"
        self.cursor.execute(query)
        self.mysql_db.commit()

    def get_result(self, cmd):
        query = f"SELECT result FROM {CMD_LOG} WHERE etl_cmd = '{cmd}'"
        self.cursor.execute(query)
        return self.cursor.fetchone()[0]

    def can_execute(self, cmd):
        query = f"SELECT c.* " \
                f"FROM {CONFIG_LOG} c " \
                f"LEFT JOIN (SELECT etl_cmd FROM {CMD_LOG} WHERE result) r " \
                f"ON c.prev_step = r.etl_cmd " \
                f"WHERE c.etl_cmd = '{cmd}' AND r.etl_cmd is null"
        self.cursor.execute(query)
        if len(self.cursor.fetchall()) == 0:
            return True

    def mysql_disconnect(self):
        stop_mysql(self.mysql_db)
