from pyspark.sql.functions import col

from common.constants import *
from common.tools import get_spark
from dependency_analysis.da_functions import DAFunctions
from etl_functions.streams_fact import StreamTransformFunctions

spark = get_spark(STREAMS_FACT)

spark.sparkContext.addPyFile(f"{ETL_PATH}/packages.zip")

daf = DAFunctions()


def extract():
    event_ext = spark.read.load(SRC_PATH_EVENTS, format='csv', sep=",", header="true", inferSchema="true")
    event_ext.write.mode('overwrite').saveAsTable(f"{STREAMS_FACT}{EXT_OP}")
    daf.log_cmd_result(f"{STREAMS_FACT}{EXT_OP}")


def transform_load():
    event_tf = StreamTransformFunctions(spark, spark.table(f"{STREAMS_FACT}{EXT_OP}"))

    event_tf.validate()
    event_tf.eliminate_nulls()
    event_tf.compute_stream_duration()
    event_tf.add_premium_dim()

    # LOAD
    event_dim = event_tf.assign_sk()
    event_dim.write.mode('overwrite').saveAsTable(f"{STREAMS_FACT}{TL_OP}")

    daf.log_cmd_result(f"{STREAMS_FACT}{TL_OP}")

    spark.sql(f"DROP TABLE IF EXISTS {STREAMS_FACT}{EXT_OP}")


def swap():
    if spark.sql(f"show tables in {DWH_DB}").filter(col('tableName') == STREAMS_FACT).count() > 0:
        spark.sql(f"ALTER TABLE {STREAMS_FACT} RENAME TO {STREAMS_FACT}{TEMP}")
    spark.sql(f"ALTER TABLE {STREAMS_FACT}{TL_OP} RENAME TO {STREAMS_FACT}")
    spark.sql(f"DROP TABLE IF EXISTS {STREAMS_FACT}{TEMP}")

    daf.log_cmd_result(f"{STREAMS_FACT}{SWAP_OP}")


def main():
    if not daf.get_result(f"{STREAMS_FACT}{EXT_OP}") and daf.can_execute(f"{STREAMS_FACT}{EXT_OP}"):
        extract()
    if not daf.get_result(f"{STREAMS_FACT}{TL_OP}") and daf.can_execute(f"{STREAMS_FACT}{TL_OP}"):
        transform_load()
    if not daf.get_result(f"{STREAMS_FACT}{SWAP_OP}") and daf.can_execute(f"{STREAMS_FACT}{SWAP_OP}"):
        swap()


main()

daf.mysql_disconnect()
