from pyspark.sql.functions import col

from common.constants import *
from common.tools import get_spark
from dependency_analysis.da_functions import DAFunctions
from etl_functions.track_dim import TrackTransformFunctions

spark = get_spark(TRACK_DIM)

spark.sparkContext.addPyFile(f"{ETL_PATH}/packages.zip")

daf = DAFunctions()


def extract():
    track_ext = spark.read.load(SRC_PATH_TRACK, format='csv', sep=",", header="true", inferSchema="true")
    track_ext.write.mode('overwrite').saveAsTable(f"{TRACK_DIM}{EXT_OP}")

    album_ext = spark.read.load(SRC_PATH_ALBUM, format='csv', sep=",", header="true", inferSchema="true")
    album_ext.write.mode('overwrite').saveAsTable(f"{ALBUM_SRC}{EXT_OP}")

    daf.log_cmd_result(f"{TRACK_DIM}{EXT_OP}")


def transform_load():
    track_tf = TrackTransformFunctions(spark, spark.table(f"{TRACK_DIM}{EXT_OP}"), spark.table(f"{ALBUM_SRC}{EXT_OP}"))

    track_tf.join_albums()
    track_tf.eliminate_nulls()
    track_tf.validate()

    if spark.sql(f"show tables in {DWH_DB}").filter(col('tableName') == TRACK_DIM).count() == 0:
        track_dim = track_tf.prep_init_form()
    else:
        track_dim = track_tf.prep_scd_form()

    track_dim.write.mode('overwrite').saveAsTable(f"{TRACK_DIM}{TL_OP}")
    daf.log_cmd_result(f"{TRACK_DIM}{TL_OP}")

    spark.sql(f"DROP TABLE IF EXISTS {TRACK_DIM}{EXT_OP}")
    spark.sql(f"DROP TABLE IF EXISTS {ALBUM_SRC}{EXT_OP}")


def swap():
    if spark.sql(f"show tables in {DWH_DB}").filter(col('tableName') == TRACK_DIM).count() > 0:
        spark.sql(f"ALTER TABLE {TRACK_DIM} RENAME TO {TRACK_DIM}{TEMP}")
    spark.sql(f"ALTER TABLE {TRACK_DIM}{TL_OP} RENAME TO {TRACK_DIM}")
    spark.sql(f"DROP TABLE IF EXISTS {TRACK_DIM}{TEMP}")

    daf.log_cmd_result(f"{TRACK_DIM}{SWAP_OP}")


def main():
    if not daf.get_result(f"{TRACK_DIM}{EXT_OP}"):
        extract()
    if not daf.get_result(f"{TRACK_DIM}{TL_OP}") and daf.can_execute(f"{TRACK_DIM}{TL_OP}"):
        transform_load()
    if not daf.get_result(f"{TRACK_DIM}{SWAP_OP}") and daf.can_execute(f"{TRACK_DIM}{SWAP_OP}"):
        swap()


main()

daf.mysql_disconnect()
