from pyspark.sql.functions import col

from common.constants import *
from common.tools import get_spark
from dependency_analysis.da_functions import DAFunctions
from etl_functions.user_dim import UserTransformFunctions

spark = get_spark(USER_DIM)

spark.sparkContext.addPyFile(f"{ETL_PATH}/packages.zip")

daf = DAFunctions()


def extract():
    user_ext = spark.read.load(SRC_PATH_USER, format='csv', sep=",", header="true", inferSchema="true")
    user_ext.write.mode('overwrite').saveAsTable(f"{USER_DIM}{EXT_OP}")
    daf.log_cmd_result(f"{USER_DIM}{EXT_OP}")


def transform_load():
    user_tf = UserTransformFunctions(spark, spark.table(f"{USER_DIM}{EXT_OP}"))

    user_tf.assign_region()
    user_tf.eliminate_nulls()
    user_tf.translate_gender()
    user_tf.validate()

    # LOAD
    if spark.sql(f"show tables in {DWH_DB}").filter(col('tableName') == USER_DIM).count() == 0:
        user_dim = user_tf.prep_init_form()
    else:
        user_dim = user_tf.prep_scd_form()

    user_dim.write.mode('overwrite').saveAsTable(f"{USER_DIM}{TL_OP}")
    daf.log_cmd_result(f"{USER_DIM}{TL_OP}")

    spark.sql(f"DROP TABLE IF EXISTS {USER_DIM}{EXT_OP}")


def swap():
    if spark.sql(f"show tables in {DWH_DB}").filter(col('tableName') == USER_DIM).count() > 0:
        spark.sql(f"ALTER TABLE {USER_DIM} RENAME TO {USER_DIM}{TEMP}")
    spark.sql(f"ALTER TABLE {USER_DIM}{TL_OP} RENAME TO {USER_DIM}")
    spark.sql(f"DROP TABLE IF EXISTS {USER_DIM}{TEMP}")

    daf.log_cmd_result(f"{USER_DIM}{SWAP_OP}")


def main():
    if not daf.get_result(f"{USER_DIM}{EXT_OP}"):
        extract()
    if not daf.get_result(f"{USER_DIM}{TL_OP}") and daf.can_execute(f"{USER_DIM}{TL_OP}"):
        transform_load()
    if not daf.get_result(f"{USER_DIM}{SWAP_OP}") and daf.can_execute(f"{USER_DIM}{SWAP_OP}"):
        swap()


main()

daf.mysql_disconnect()
