from common.constants import *


class Calendar:
    def __init__(self, spark):
        self.spark = spark

    def create(self):
        self.spark.sql(
            f"SELECT explode(sequence(to_date('{CALENDAR_START_DATE}'), to_date('{CALENDAR_END_DATE}'), interval 1 day)) AS date "
        ).createOrReplaceTempView('dates')

        calendar_dim = self.spark.sql(
            "SELECT "
            "INT(year(date)*10000+month(date)*100+day(date)) as date_id, "
            "date_format(date, 'yyyy-MM-dd') AS `date`, "
            "STRING(date_format(date, 'yyyy')) AS year, "
            "STRING(date_format(date, 'MMMM')) AS month_full, "
            "STRING(date_format(date,'MMM')) AS month_short, "
            "STRING(date_format(date, 'MM')) AS month_number, "
            "STRING(date_format(date, 'dd')) AS day_of_month, "
            "STRING(date_format(date, 'EEEE')) AS day_of_week_full, "
            "STRING(date_format(date, 'EEE')) AS day_of_week_short, "
            "STRING(weekday(date)+1) AS day_of_week_number, "
            "STRING(weekofyear(date)) AS week_of_year "
            "FROM dates"
        )

        calendar_dim = self.get_null_row().union(calendar_dim)

        calendar_dim.write \
            .mode('overwrite') \
            .saveAsTable(CALENDAR_DIM)

    def get_null_row(self):
        columns = ['date_id', 'date', 'year', 'month_full', 'month_short', 'month_number', 'day_of_month',
                   'day_of_week_full', 'day_of_week_short', 'day_of_week_number', 'week_of_year']
        vals = [(-1,) + (('unknown',) * 10)]
        return self.spark.createDataFrame(vals, columns)
