import datetime
import subprocess
import time

from common.constants import ETL_PATH, NAIVE, USER_DIM_JOB, STREAM_FACT_JOB, TRACK_DIM_JOB
from common.tools import get_logger


class NaiveMethod:
    def __init__(self):
        self.logger = get_logger(f"method_{NAIVE}")
        self.fail_id = -1
        self.timeout = -1
        self.start_time = None

    def submit_job(self, job, job_id=0):
        command = ['spark-submit', '--py-files', f'{ETL_PATH}/packages.zip', f"{ETL_PATH}/{NAIVE}/{job}"]
        return self.run(command, fail=self.fail_id == job_id)

    def run(self, command, fail=False):
        process = subprocess.Popen(command, stdout=subprocess.PIPE, stderr=subprocess.PIPE, universal_newlines=True)

        if fail:
            try:
                process.communicate(timeout=self.timeout)
                return 0
            except subprocess.TimeoutExpired:
                process.terminate()
                self.logger.error("Planned error")
                return 1
        else:
            stdout, stderr = process.communicate()
            if process.returncode != 0:
                self.logger.error(stderr)
            return process.returncode

    def ETL(self):
        result = 0
        result += self.submit_job(USER_DIM_JOB, job_id=101)
        result += self.submit_job(TRACK_DIM_JOB, job_id=102)
        result += 1 if result > 0 else self.submit_job(STREAM_FACT_JOB, job_id=103)

        if result != 0:
            self.resumption()

    def resumption(self):
        self.fail_id = -1

        self.ETL()

    def start(self, fail_id, timeout):
        self.fail_id = fail_id
        self.timeout = timeout
        self.start_time = datetime.datetime.now()

        self.ETL()

        return self.get_runtime()

    def get_runtime(self):
        return time.strftime('%H:%M:%S',
                             time.gmtime((datetime.datetime.now() - self.start_time).seconds))
