from pyspark.sql.functions import col

from common.constants import *
from common.tools import get_spark
from etl_functions.streams_fact import StreamTransformFunctions

spark = get_spark(STREAMS_FACT)

spark.sparkContext.addPyFile(f"{ETL_PATH}/packages.zip")


def main():
    event_ext = spark.read.load(SRC_PATH_EVENTS, format='csv', sep=",", header="true", inferSchema="true")

    event_tf = StreamTransformFunctions(spark, event_ext)
    event_tf.validate()
    event_tf.eliminate_nulls()
    event_tf.compute_stream_duration()
    event_tf.add_premium_dim()

    # LOAD
    event_dim = event_tf.assign_sk()
    if spark.sql(f"show tables in {DWH_DB}").filter(col('tableName') == STREAMS_FACT).count() == 0:
        event_dim.write.mode('overwrite').saveAsTable(f"{STREAMS_FACT}")
    else:
        event_dim.write.mode('overwrite').saveAsTable(f"{STREAMS_FACT}{TEMP}")
        spark.sql(f"ALTER TABLE {STREAMS_FACT} RENAME TO {STREAMS_FACT}{BACKUP}")
        spark.sql(f"ALTER TABLE {STREAMS_FACT}{TEMP} RENAME TO {STREAMS_FACT}")
        spark.sql(f"DROP TABLE IF EXISTS {STREAMS_FACT}{BACKUP}")


main()
