from pyspark.sql.functions import col

from common.constants import *
from common.tools import get_spark
from etl_functions.user_dim import UserTransformFunctions

spark = get_spark(USER_DIM)

spark.sparkContext.addPyFile(f"{ETL_PATH}/packages.zip")


def main():
    user_ext = spark.read.load(SRC_PATH_USER, format='csv', sep=",", header="true", inferSchema="true")

    user_tf = UserTransformFunctions(spark, user_ext)

    user_tf.assign_region()
    user_tf.eliminate_nulls()
    user_tf.translate_gender()
    user_tf.validate()

    # LOAD
    if spark.sql(f"show tables in {DWH_DB}").filter(col('tableName') == USER_DIM).count() == 0:
        user_dim = user_tf.prep_init_form()
        user_dim.write.mode("overwrite").saveAsTable(USER_DIM)
    else:
        user_dim = user_tf.prep_scd_form()

        user_dim.write.mode('overwrite').saveAsTable(f"{USER_DIM}{TEMP}")
        spark.sql(f"ALTER TABLE {USER_DIM} RENAME TO {USER_DIM}{BACKUP}")
        spark.sql(f"ALTER TABLE {USER_DIM}{TEMP} RENAME TO {USER_DIM}")
        spark.sql(f"DROP TABLE IF EXISTS {USER_DIM}{BACKUP}")


main()
