import logging

from pyspark.sql import SparkSession

from common.constants import MASTER_ADDR, LOG_FILE


logger: logging.Logger = None


def get_spark():
    sparkSession = SparkSession.builder \
        .appName("src") \
        .master(MASTER_ADDR) \
        .getOrCreate()

    return sparkSession


def setup_logger():
    formatter = logging.Formatter('%(asctime)s %(levelname)s: %(message)s',
                                  '%Y-%m-%d %H:%M:%S')
    handler = logging.FileHandler(LOG_FILE)
    handler.setFormatter(formatter)

    log = logging.getLogger('logger')
    log.setLevel(logging.INFO)
    log.addHandler(handler)
    return log


def get_logger():
    global logger
    if logger is None:
        logger = setup_logger()
    return logger
