import pandas as pd

from common.functions import read, write

from common.constants import *
from common.tools import get_logger
from datasets_gen.gen_events import generate_events
from datasets_gen.gen_users import generate_users

logger = get_logger()


def get_tracks(n):
    tracks_ext = read(TRACK_EXT_DATASET)
    tracks_ext_sample = tracks_ext.sample(n) if n != -1 else tracks_ext

    tracks_sample = tracks_ext_sample[['title', 'artist', 'album', 'duration_ms', 'genre']]
    albums_sample = tracks_ext_sample[['album', 'artist', 'release_date', 'publisher']]
    albums_sample.drop_duplicates(['album', 'artist'], inplace=True)
    albums_sample.rename({'album': 'title'}, inplace=True, axis=1)
    tracks_sample.drop_duplicates(['title', 'artist', 'album'], inplace=True)

    write(tracks_sample, TRACK_OUT)
    write(albums_sample, ALBUM_OUT)

    logger.info(f"Got {tracks_sample.shape[0]} tracks from {albums_sample.shape[0]} albums")
    return tracks_sample


def create_events(n, users, tracks):
    write(generate_events(n, users, tracks), EVENT_OUT)


def add_events(n, users, tracks):
    events = read(EVENT_OUT)
    events_new = generate_events(n, users, tracks)
    events_all = pd.concat([events, events_new])

    write(events_all, EVENT_OUT)
    logger.info(f"Added {n * 2} new events. All events: {events_all.shape[0]}")


def create_users(n):
    users = generate_users(n)
    write(users, USER_OUT)
    return users


def add_users(n):
    users = read(USER_OUT)
    users_new = generate_users(n)
    users_all = pd.concat([users, users_new])

    users_all.drop_duplicates(subset=['email'], keep='first', inplace=True)

    write(users_all, USER_OUT)
    logger.info(f"Added {n} new users. All users: {users_all.shape[0]}")

    return users_all


def create_datasets(users, tracks, events):
    users = create_users(users)
    tracks = get_tracks(tracks)
    create_events(events, users, tracks)


def add_new(users, events):
    users = add_users(users)
    tracks = read(TRACK_OUT)
    add_events(events, users, tracks)

