import datetime
import random
import time
from logging import Logger

import pandas as pd

from common.constants import CODE_PATH
from common.tools import get_logger

pd.options.mode.chained_assignment = None

logger: Logger = get_logger()


def get_current_date(inc):
    with open(f'{CODE_PATH}/calendar', 'r') as cal:
        date = datetime.datetime.strptime(cal.readline(), '%Y-%m-%d %H:%M:%S')
    if inc:
        with open(f'{CODE_PATH}/calendar', 'w') as cal:
            cal.write((date + datetime.timedelta(days=1)).strftime('%Y-%m-%d %H:%M:%S'))
    return date


def get_random_ms(dur):
    percentage_played = random.randint(5, 100)
    ms_played = int((percentage_played * dur) / 100)
    return ms_played if ms_played > 1000 else 1000


def get_timestamp(date):
    return (date + datetime.timedelta(hours=random.randint(0, 23),
                                      minutes=random.randint(0, 59),
                                      seconds=random.randint(0, 59)))


def generate_events(n, users, tracks):
    logger.info(f"Generating {n} events")

    start_time = datetime.datetime.now()
    date = get_current_date(True)

    tracks = tracks.sample(n, replace=True)
    events_df = users.sample(n, replace=True)[['email']]
    events_df['track'] = tracks['title'].values
    events_df['artist'] = tracks['artist'].values
    events_df['album'] = tracks['album'].values
    events_df['dur'] = tracks['duration_ms'].values

    events_df['play_time'] = events_df.apply(lambda row: get_timestamp(date), axis=1)
    events_df['pause_time'] = events_df.apply(
        lambda row: (row['play_time'] + datetime.timedelta(milliseconds=get_random_ms(row['dur']))).strftime(
            '%Y-%m-%d %H:%M:%S'), axis=1)
    events_df['play_time'] = events_df.apply(lambda row: row['play_time'].strftime('%Y-%m-%d %H:%M:%S'), axis=1)

    events_df.drop('dur', axis=1, inplace=True)

    events_df_play = events_df[['email', 'track', 'artist', 'album', 'play_time']]
    events_df_pause = events_df[['email', 'track', 'artist', 'album', 'pause_time']]

    events_df_play['action'] = 'play'
    events_df_pause['action'] = 'pause'

    events_df_play.rename({'play_time': 'datetime'}, inplace=True, axis=1)
    events_df_pause.rename({'pause_time': 'datetime'}, inplace=True, axis=1)

    events_df = pd.concat([events_df_play, events_df_pause])
    events_df.sort_values(by=['datetime'], inplace=True)

    events_df = events_df[['datetime', 'email', 'track', 'artist', 'album', 'action']]

    logger.info("Created {num} events in {time}"
                .format(now=datetime.datetime.now().strftime('%H:%M:%S'),
                        num=events_df.shape[0],
                        time=time.strftime('%H:%M:%S',
                                           time.gmtime((datetime.datetime.now() - start_time).seconds))))

    return events_df
