import datetime
import random
import time
from logging import Logger

from dateutil.relativedelta import relativedelta

from common.constants import *
from common.functions import read
from common.tools import get_logger

logger: Logger = get_logger()


def get_email(e_dom, firstname, lastname):
    email = random.choice(EMAIL_TEMPLATES)
    email = email.replace("_fn_", firstname.lower()) if email.find("_fn_") > -1 else email
    email = email.replace("_ln_", lastname.lower()) if email.find("_ln_") > -1 else email
    email = email.replace("_sfn_", firstname.lower()[0]) if email.find("_sfn_") > -1 else email
    email = email.replace("_sln_", lastname.lower()[0]) if email.find("_sln_") > -1 else email
    if email.find("_num_") > 1:
        len_n = random.randint(1, 3)
        num = ""
        for i in range(len_n):
            num += str(random.randrange(9))
        email = email.replace("_num_", num)

    email = email.replace("_dom_", e_dom)
    return email


def get_birthday():
    age_options = [x for x in range(15, 80, 1)]
    year = random.choice(age_options)

    date_r = random.randint(int(datetime.datetime.strptime('2022-01-01', '%Y-%m-%d').timestamp()),
                            int(datetime.datetime.strptime('2022-12-31', '%Y-%m-%d').timestamp()))

    return (datetime.date.fromtimestamp(date_r) - relativedelta(years=year)).strftime('%Y-%m-%d')


def generate_users(n):
    logger.info(f"Generating {n} users")

    start_time = datetime.datetime.now()

    users_df = read(FIRSTNAMES_DATASET).sample(n, replace=True)
    users_ln = read(LASTNAMES_DATASET).sample(n, replace=True)
    users_streets = read(STREETS_DATASET).sample(n, replace=True)
    users_countries = read(COUNTRIES_DATASET).sample(n, replace=True)
    e_domains = read(EMAIL_DOMAINS_DATASET).sample(n, replace=True)

    users_df['lastname'] = users_ln['lastname'].values
    users_df['address'] = users_streets['street'].values
    users_df['zip_code'] = users_streets['postcode'].values
    users_df['city'] = users_countries['city_name'].values
    users_df['country'] = users_countries['country_name'].values
    users_df['domain'] = e_domains['domain'].values

    users_df['email'] = users_df.apply(lambda row: get_email(row['domain'], row['firstname'], row['lastname']), axis=1)

    users_df['birthday'] = users_df.apply(lambda row: get_birthday(), axis=1)

    users_df['address'] = users_df.apply(lambda row: f"{row['address']} {random.randint(1, 300)}", axis=1)

    users_df['premium'] = users_df.apply(lambda row: random.choice([True, False]), axis=1)

    users_df.rename({'firstname': 'name'}, inplace=True, axis=1)

    users_df = users_df[['email', 'name', 'birthday', 'gender', 'address', 'city', 'zip_code', 'country', 'premium']]

    users_df.drop_duplicates(subset=['email'], keep='first', inplace=True)

    logger.info("Created {num} users in {time}"
                .format(now=datetime.datetime.now().strftime('%H:%M:%S'),
                        num=users_df.shape[0],
                        time=time.strftime('%H:%M:%S',
                                           time.gmtime((datetime.datetime.now() - start_time).seconds))))

    return users_df
